]> sigrok.org Git - libsigrok.git/blobdiff - std.c
Remove SR_INST_USBTMC which is no longer used.
[libsigrok.git] / std.c
diff --git a/std.c b/std.c
index 5eb2e57d8e9808f19c1e13588484f92ead7592d4..ed37abeac641b02a760f6e1d46bad285772c002e 100644 (file)
--- a/std.c
+++ b/std.c
@@ -1,5 +1,5 @@
 /*
- * This file is part of the sigrok project.
+ * This file is part of the libsigrok project.
  *
  * Copyright (C) 2013 Uwe Hermann <uwe@hermann-uwe.de>
  *
@@ -25,7 +25,7 @@
 /**
  * Standard sr_driver_init() API helper.
  *
- * This function can be used to simplify most driver's hw_init() API callback.
+ * This function can be used to simplify most driver's init() API callback.
  *
  * It creates a new 'struct drv_context' (drvc), assigns sr_ctx to it, and
  * then 'drvc' is assigned to the 'struct sr_dev_driver' (di) that is passed.
@@ -37,8 +37,8 @@
  * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
  *         SR_ERR_MALLOC upon memory allocation errors.
  */
-SR_PRIV int std_hw_init(struct sr_context *sr_ctx, struct sr_dev_driver *di,
-                       const char *prefix)
+SR_PRIV int std_init(struct sr_context *sr_ctx, struct sr_dev_driver *di,
+                    const char *prefix)
 {
        struct drv_context *drvc;
 
@@ -47,12 +47,13 @@ SR_PRIV int std_hw_init(struct sr_context *sr_ctx, struct sr_dev_driver *di,
                return SR_ERR_ARG;
        }
 
-       if (!(drvc = g_try_malloc0(sizeof(struct drv_context)))) {
+       if (!(drvc = g_try_malloc(sizeof(struct drv_context)))) {
                sr_err("%sDriver context malloc failed.", prefix);
                return SR_ERR_MALLOC;
        }
 
        drvc->sr_ctx = sr_ctx;
+       drvc->instances = NULL;
        di->priv = drvc;
 
        return SR_OK;
@@ -62,7 +63,7 @@ SR_PRIV int std_hw_init(struct sr_context *sr_ctx, struct sr_dev_driver *di,
  * Standard API helper for sending an SR_DF_HEADER packet.
  *
  * This function can be used to simplify most driver's
- * hw_dev_acquisition_start() API callback.
+ * dev_acquisition_start() API callback.
  *
  * @param sdi The device instance to use.
  * @param prefix A driver-specific prefix string used for log messages.
@@ -100,27 +101,31 @@ SR_PRIV int std_session_send_df_header(const struct sr_dev_inst *sdi,
        return SR_OK;
 }
 
+#ifdef HAVE_LIBSERIALPORT
+
 /*
  * Standard sr_session_stop() API helper.
  *
  * This function can be used to simplify most (serial port based) driver's
- * hw_dev_acquisition_stop() API callback.
+ * dev_acquisition_stop() API callback.
  *
  * @param sdi The device instance for which acquisition should stop.
  *            Must not be NULL.
  * @param cb_data Opaque 'cb_data' pointer. Must not be NULL.
* @param hw_dev_close_fn Function pointer to the driver's hw_dev_close().
  *                       Must not be NULL.
  * @param serial The serial device instance (struct serial_dev_inst *).
  *               Must not be NULL.
  * @param prefix A driver-specific prefix string used for log messages.
  *               Must not be NULL. An empty string is allowed.
  *
* @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
*         SR_ERR upon other errors.
  */
-SR_PRIV int std_hw_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
-                       void *cb_data, dev_close_t hw_dev_close_fn,
+SR_PRIV int std_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
+                       void *cb_data, dev_close_t dev_close_fn,
                        struct sr_serial_dev_inst *serial, const char *prefix)
 {
        int ret;
@@ -133,7 +138,7 @@ SR_PRIV int std_hw_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
 
        if (sdi->status != SR_ST_ACTIVE) {
                sr_err("%sDevice inactive, can't stop acquisition.", prefix);
-               return SR_ERR;
+               return SR_ERR_DEV_CLOSED;
        }
 
        sr_dbg("%sStopping acquisition.", prefix);
@@ -143,7 +148,7 @@ SR_PRIV int std_hw_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
                return ret;
        }
 
-       if ((ret = hw_dev_close_fn(sdi)) < 0) {
+       if ((ret = dev_close_fn(sdi)) < 0) {
                sr_err("%sFailed to close device: %d.", prefix, ret);
                return ret;
        }
@@ -159,3 +164,69 @@ SR_PRIV int std_hw_dev_acquisition_stop_serial(struct sr_dev_inst *sdi,
 
        return SR_OK;
 }
+
+#endif
+
+/*
+ * Standard driver dev_clear() helper.
+ *
+ * This function can be used to implement the dev_clear() driver API
+ * callback. dev_close() is called before every sr_dev_inst is cleared.
+ *
+ * The only limitation is driver-specific device contexts (sdi->priv).
+ * These are freed, but any dynamic allocation within structs stored
+ * there cannot be freed.
+ *
+ * @param driver The driver which will have its instances released.
+ * @param clear_private If not NULL, this points to a function called
+ * with sdi->priv as argument. The function can then clear any device
+ * instance-specific resources kept there. It must also clear the struct
+ * pointed to by sdi->priv.
+ *
+ * @return SR_OK on success.
+ */
+SR_PRIV int std_dev_clear(const struct sr_dev_driver *driver,
+               std_dev_clear_t clear_private)
+{
+       struct drv_context *drvc;
+       struct sr_dev_inst *sdi;
+       GSList *l;
+       int ret;
+
+       if (!(drvc = driver->priv))
+               /* Driver was never initialized, nothing to do. */
+               return SR_OK;
+
+       ret = SR_OK;
+       for (l = drvc->instances; l; l = l->next) {
+               if (!(sdi = l->data)) {
+                       ret = SR_ERR_BUG;
+                       continue;
+               }
+               if (driver->dev_close)
+                       driver->dev_close(sdi);
+
+               if (sdi->conn) {
+#if HAVE_LIBSERIALPORT
+                       if (sdi->inst_type == SR_INST_SERIAL)
+                               sr_serial_dev_inst_free(sdi->conn);
+#endif
+#if HAVE_LIBUSB_1_0
+                       if (sdi->inst_type == SR_INST_USB)
+                               sr_usb_dev_inst_free(sdi->conn);
+#endif
+                       if (sdi->inst_type == SR_INST_SCPI)
+                               sr_scpi_free(sdi->conn);
+               }
+               if (clear_private)
+                       clear_private(sdi->priv);
+               else
+                       g_free(sdi->priv);
+               sr_dev_inst_free(sdi);
+       }
+
+       g_slist_free(drvc->instances);
+       drvc->instances = NULL;
+
+       return ret;
+}
@@ -100,27 +101,31 @@ SR_PRIV int std_session_send_df_header(const struct sr_dev_inst *sdi,
  *            Must not be NULL.
  * @param cb_data Opaque 'cb_data' pointer. Must not be NULL.
- * @param hw_dev_close_fn Function pointer to the driver's hw_dev_close().
+ * @param dev_close_fn Function pointer to the driver's dev_close().
  *                       Must not be NULL.
  * @param serial The serial device instance (struct serial_dev_inst *).
@@ -100,27 +101,31 @@ SR_PRIV int std_session_send_df_header(const struct sr_dev_inst *sdi,
  *               Must not be NULL. An empty string is allowed.
  *
- * @return SR_OK upon success, SR_ERR_ARG upon invalid arguments, or
- *         SR_ERR upon other errors.
+ * @retval SR_OK Success.
+ * @retval SR_ERR_ARG Invalid arguments.
+ * @retval SR_ERR_DEV_CLOSED Device is closed.
+ * @retval SR_ERR Other errors.
  */